package com.bytesw.bytebot.service;

import com.bytesw.bytebot.http.FileValidationResponse;
import com.bytesw.bytebot.http.FileValidationResult;
import com.bytesw.bytebot.http.enums.ValidationStatusEnum;
import com.bytesw.bytebot.model.BdcControl;
import com.bytesw.bytebot.model.QuestionFile;
import com.bytesw.bytebot.model.enums.FrequentQuestionStatusEnum;
import com.bytesw.bytebot.model.enums.StatusBcdEnum;
import com.bytesw.bytebot.repository.BdcControlRepository;
import com.bytesw.bytebot.repository.QuestionFileRepository;
import com.bytesw.xdf.exception.NotFoundException;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Optional;

@Service
@Transactional
@Log4j2
public class OrquestadorService {

    @Autowired
    private FileManagementService fileManagementService;

    @Autowired
    private BucketService bucketService;

    @Autowired
    private BdcControlRepository bdcControlRepository;

    @Autowired
    private QuestionFileRepository questionFileRepository;

    @Autowired
    private  KnowledgeService knowledgeService;


    public FileValidationResponse executeGenerateBCD(String uuid, MultipartFile file) {
        BdcControl model = new BdcControl();
        FileValidationResponse response = new FileValidationResponse();
        boolean step = false;
        try{
            //Valida y guarda en base de datos
            response = fileManagementService.validateAndSaveFile(uuid,file);
            if(response.getFileValidationResult().getStatus().getName() != ValidationStatusEnum.OK.getName()) {
                response.setStatus(FrequentQuestionStatusEnum.ERROR.getName());
                return response;
            }
            model.setUuid(response.getUuid());
            model.setFileId(response.getId());
            model.setFileName(response.getFileName());
            model.setDate(response.getUploadDate());
            model.setStatus(StatusBcdEnum.CARGADO.getName());
            model = bdcControlRepository.save(model);

            //Carga archivo a bucket
            step = bucketService.generate(file.getBytes(), response);
            if (!step) {
                throw new NotFoundException("Error a cargar archivo.");
            }
            model.setStatus(StatusBcdEnum.INDEXANDO.getName());
            model = bdcControlRepository.save(model);

            //Generacion del modelo
            //Actualizar
            knowledgeService.generate(response);
            response.setStatus(FrequentQuestionStatusEnum.CORRECTO.getName());
            model.setStatus(StatusBcdEnum.INDEXADO.getName());
            model = bdcControlRepository.save(model);
        }catch (Exception e) {
            response.setStatus(FrequentQuestionStatusEnum.ERROR.getName());
        }
        return response;
    }

    public FileValidationResponse retry(String uuid) {
        Optional<BdcControl> modelBdc = bdcControlRepository.findByUuid(uuid);
        if (!modelBdc.isPresent()) {
            throw new NotFoundException("No se subio archivo a la BD");
        }
        boolean step = false;
        BdcControl model = modelBdc.get();
        FileValidationResponse response = modelToFileValidation(model);
        try {
            if (model.getStatus().equals(StatusBcdEnum.CARGADO.getName())) {
                Optional<QuestionFile> questionFile = questionFileRepository.findByUuid(uuid);
                if (!questionFile.isPresent()) {
                    throw new NotFoundException("No se encontro archivo en la BD");
                }
                step = bucketService.generate(questionFile.get().getData(), response);
                if (!step) {
                    throw new NotFoundException("Error a cargar archivo.");
                }
                model.setStatus(StatusBcdEnum.INDEXANDO.getName());
                model = bdcControlRepository.save(model);
                step = false;
            }
            if (model.getStatus().equals(StatusBcdEnum.INDEXANDO.getName())) {
                step = knowledgeService.generate(response);
                if (!step) {
                    throw new NotFoundException("Error a cargar knowledge.");
                }
            }
            model.setStatus(StatusBcdEnum.INDEXADO.getName());
            model = bdcControlRepository.save(model);
            response.setStatus(FrequentQuestionStatusEnum.CORRECTO.getName());
        } catch (IOException e) {
            response.setStatus(FrequentQuestionStatusEnum.ERROR.getName());
        }
        return response;
    }

    public void deleteBcd(String uuid) {
        Optional<BdcControl> bdcControl = bdcControlRepository.findByUuid(uuid);
        if (!bdcControl.isPresent()) {
            throw new NotFoundException("Control no encontrado.");
        }
        Optional<QuestionFile> questionFile = questionFileRepository.findByUuid(uuid);
        if (questionFile.isPresent()) {
            questionFileRepository.deleteById(bdcControl.get().getId());
            bdcControl.get().setStatus(StatusBcdEnum.INDEXANDO.getName());
        }

        if (bdcControl.get().getStatus().equals(StatusBcdEnum.INDEXANDO.getName())) {
            bucketService.delete(uuid);
            bdcControl.get().setStatus(StatusBcdEnum.INDEXADO.getName());
        }
        if (bdcControl.get().getStatus().equals(StatusBcdEnum.INDEXADO.getName())) {
            knowledgeService.delete(uuid);
        }
        bdcControlRepository.deleteById(bdcControl.get().getId());
    }

    private FileValidationResponse modelToFileValidation(BdcControl model){
        FileValidationResponse response = new FileValidationResponse();
        response.setId(model.getId());
        response.setStatus(model.getStatus());
        response.setUuid(model.getUuid());
        response.setFileName(model.getFileName());
        response.setUploadDate(model.getDate());
        FileValidationResult result = new FileValidationResult();
        result.setStatus(ValidationStatusEnum.OK);
        response.setFileValidationResult(result);
        return response;
    }

}
